# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Name:         game_loop.py
# Author:       Pascal Peter
# Copyright:    (C) 2003-2013 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Execution du programme.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import sys
import os
import time
import subprocess

import bge

HERE = os.path.abspath(bge.logic.expandPath('//'))
scriptsDir = os.path.join(HERE, 'scripts')
sys.path.insert(0, scriptsDir)

# modules persos placés dans le sous-dossier scripts :
import utils, utils_i18n



# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = bge.logic.getCurrentController()
owner = controller.owner
scene = bge.logic.getCurrentScene()
camera = scene.active_camera



# **************************************************
#       CONSTANTES DIVERSES :
# **************************************************
MESSAGE_KEY = bge.events.MKEY
LANG = 'fr'


# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postText():
    if bge.logic.globalDict['STATE'] != 'game_loop':
        return
    font = bge.logic.globalDict['fonts'][1]
    # les textes affichés à l'écran :
    for textName in bge.logic.globalDict['TEXTS']:
        params = bge.logic.globalDict['TEXTS'][textName]
        utils.print2D(params[0], params[4], params[2], params[3], font)

def gestKeyboard():
    """
    gestion du clavier
    """
    keyboard = bge.logic.keyboard.events
    JUST_ACTIVATED = bge.logic.KX_INPUT_JUST_ACTIVATED
    # touches spéciales :
    if keyboard[MESSAGE_KEY] == JUST_ACTIVATED:
        # pour écrire un message
        doMessage()

def doMessage():
    """
    aaa
    """
    messageText = ''
    args = []
    root = bge.logic.expandPath('//')
    if sys.platform.startswith('win'):
        # vu que sous windows Python n'est pas forcément installé,
        # j'ai dû faire un exe :
        processFile = '{0}scripts{1}winexe{1}messages.exe'.format(root, os.sep)
        processFile = bge.logic.expandPath(processFile)
        args = [processFile, 'LANG={0}'.format(LANG)]
    else:
        # on lance le fichier messages.py :
        processFile = '{0}scripts{1}messages.py'.format(root, os.sep)
        processFile = bge.logic.expandPath(processFile)
        args = ['python', processFile, 'LANG={0}'.format(LANG)]
    try:
        process = subprocess.Popen(args)
        bge.logic.globalDict['pause'] = True
        while process.poll() == None:
            time.sleep(0.1)
        del process
        messageText = utils.loadMessageText()
    except:
        print('SERVER EXECUTION FAILED')
    if messageText != '':
        messageText = messageText.replace('\n', ' NEWLINE ')
        utils.setText('myText', text=messageText)
    bge.logic.globalDict['pause'] = False


def gestCharacter():
    """
    On teste les déplacements à chaque boucle :
    """
    if bge.logic.globalDict['STATE'] != 'game_loop':
        return
    cube = scene.objects['Cube']
    cube.applyRotation([0, 0, 0.1], True)


# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    if not('STATE' in bge.logic.globalDict):
        bge.logic.globalDict['STATE'] = 'game_first'
    state = bge.logic.globalDict['STATE']
    if not(state in ('game_first', 'game_loop', 'game_quit')):
        bge.logic.globalDict['STATE'] = 'game_first'

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['STATE'] == 'game_first':
        # affichage des textes :
        utils.checkFont()
        scene.post_draw += [postText]
        bge.logic.globalDict['TEXTS'] = {}
        helpText = utils_i18n.tr('divers', 'M key to write a message', LANG)
        utils.setText('helpText', text=helpText, 
            color=utils.colors['orange'], width=2.5, position=(0.05, 0.25))
        utils.setText('myText', 
            color=utils.colors['blue'], width=2, position=(0.18, 0.4))
        # initialisation terminée :
        bge.logic.globalDict['pause'] = False
        bge.logic.globalDict['STATE'] = 'game_loop'

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['STATE'] == 'game_loop':
        if bge.logic.globalDict['pause']:
            return
        # gestion du clavier :
        gestKeyboard()
        # on teste les déplacements à chaque boucle :
        gestCharacter()

