# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Name:         utils.py
# Author:       Pascal Peter
# Copyright:    (C) 2003-2013 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Ce module contient les variables et fonctions utiles au programme.
"""

# importation des modules utiles :
import sys
import os

import bge
from bgl import *
import blf



###########################################################'
#
#       FONCTIONS DIVERSES :
#
###########################################################'

SHORTNAME = 'college3d'
def findConfigDir():
    """
    retourne l'emplacement du fichier de configuration
    """
    configDir = ''
    if sys.platform == 'win32':
        configDir = os.environ['APPDATA']
        configDir = os.path.join(configDir, SHORTNAME)
    else:
        configsDir = os.path.join(os.path.expanduser('~'), '.config')
        if os.path.isdir(configsDir):
            configDir = os.path.join(configsDir, SHORTNAME)
        else:
            configDir = os.path.join(os.path.expanduser('~'), '.{0}'.format(SHORTNAME))
    # on vérifie l'existence du dossier :
    if not(os.path.isdir(configDir)):
        os.mkdir(configDir)
    if not(os.path.isdir(configDir)):
        configDir = bge.logic.expandPath('//')
    #print('configDir :', configDir)
    return configDir



###########################################################'
#
#       AFFICHAGE DES TEXTES :
#       d'après :
#       http://www.highspeedracer.webou.net/tuto.php?id=33
#
###########################################################'
# les couleurs utilisées pour les textes :
colors = {
    'orange':   [1.0, 0.5, 0.25, 1.0], 
    'blue':     [0.0, 0.0, 1.0, 1.0], 
    'violet':   [1.0, 0.0, 1.0, 1.0],
    'white':    [1.0, 1.0, 1.0, 1.0],
    'gray':     [0.5, 0.5, 0.5, 1.0],
    }

def addFont(font) :
    bge.logic.globalDict['fonts'].append(blf.load(font))
    return len(bge.logic.globalDict['fonts']) - 1

def checkFont():
    fontName = 'Comfortaa_Bold.ttf'
    if not('fonts' in bge.logic.globalDict):
        bge.logic.globalDict['fonts'] = [0]
        root = bge.logic.expandPath("//")
        fontFile = '{1}data{0}{2}'.format(os.sep, root, fontName)
        fontFile = bge.logic.expandPath(fontFile)
        f = addFont(fontFile)
        #print('addFont :', fontFile, ' | ', f)

def print2D(text, pos, color=[0, 0, 0, 1], size=1, font=0):
    # Recupere les informations de l'affichage - Collect viewport information
    view_buf = Buffer(GL_INT, 4)
    glGetIntegerv(GL_VIEWPORT, view_buf)
    view = view_buf

    # Parametre les matrices OpenGL - Setup OpenGL matrices
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(0, view[2], 0, view[3])

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    # On desactive les ombres - Disable shadows
    glDisable(GL_LIGHTING)

    height = bge.render.getWindowHeight()
    width = bge.render.getWindowWidth()

    # Taille du texte - Text size
    blf.size(font, int(height / 20), int(50 * size))
    glColor4f(color[0], color[1], color[2], color[3])
    # On écrit le texte en le découpant en lignes :
    lines = 0
    for line in text.split(' NEWLINE '):
        if lines > 0:
            decalage = int(lines * 1.5 * height / 20)
        else:
            decalage = 0
        blf.position(font, pos[0] * width, (1 - pos[1]) * height - decalage, 0)
        blf.draw(font, line)
        lines += 1

    # On reactive les ombres - Enable shadows
    glEnable(GL_LIGHTING)

def setText(textName='text1', text=None, time=None, color=None, width=None, position=None):
    if not('TEXTS' in bge.logic.globalDict):
        bge.logic.globalDict['TEXTS'] = {}
    if textName in bge.logic.globalDict['TEXTS']:
        # on ne modifie que ce qui est passé en paramètre :
        if text == None:
            text = bge.logic.globalDict['TEXTS'][textName][0]
        if time == None:
            time = bge.logic.globalDict['TEXTS'][textName][1]
        if color == None:
            color = bge.logic.globalDict['TEXTS'][textName][2]
        if width == None:
            width = bge.logic.globalDict['TEXTS'][textName][3]
        if position == None:
            position = bge.logic.globalDict['TEXTS'][textName][4]
    else:
        # on met de vraies valeurs pour les paramètres absents :
        if text == None:
            text = ''
        if time == None:
            time = -1
        if color == None:
            color = colors['white']
        if width == None:
            width = 1
        if position == None:
            position = (0.0, 0.5)
    bge.logic.globalDict['TEXTS'][textName] = [text, time, color, width, position]




###########################################################'
#
#       RÉCUPÉRATION DU TEXTE ÉCRIT DANS MESSAGES.PY :
#
###########################################################'
def loadMessageText():
    """
    lit le contenu du fichier message.txt
    """
    messageText = ''
    try:
        messageFileName = os.path.join(findConfigDir(), 'message.txt')
        theFile = open(messageFileName, newline='', encoding='utf-8')
        messageText = ''.join(theFile.readlines())
        theFile.close()
    except:
        pass
    return messageText

