# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Name:         utils_i18n.py
# Author:       Pascal Peter
# Copyright:    (C) 2003-2013 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Traductions des textes du programme.
    On les récupère dans des fichiers csv.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import os
import csv
import bge



# **************************************************
#       LE DICTIONNAIRE DES TRADUCTIONS
# **************************************************
TRANSLATIONS = {'csvFiles':[]}

# **************************************************
#       FONCTIONS UTILES
# **************************************************
def tr(csvFile, text, lang):
    """
    traduit le texte dans la langue demandée.
    Si la traduction n'est pas trouvée, on retourne le texte de départ.
    """
    result = text
    # on vérifie que le bon fichier est ouvert, sinon on l'ouvre :
    if not(csvFile in TRANSLATIONS['csvFiles']):
        openCsvFile(csvFile)
    # on lit la traduction :
    if text in TRANSLATIONS:
        if lang in TRANSLATIONS[text]:
            result = TRANSLATIONS[text][lang]
    if result == '':
        result = text
    return result

def openCsvFile(csvFile):
    global TRANSLATIONS
    languages = []
    root = bge.logic.expandPath('//')
    csvFileName = '{1}translations{0}{2}.csv'.format(os.sep, root, csvFile)
    csvFileName = bge.logic.expandPath(csvFileName)
    try:
        TRANSLATIONS['csvFiles'].append(csvFile)
        nbCols = 0
        theFile = open(csvFileName, newline='', encoding='utf-8')
        reader = csv.reader(theFile, 
                            delimiter=';', 
                            quotechar='"', 
                            quoting=csv.QUOTE_NONNUMERIC)
        firstRow = True
        for row in reader:
            if firstRow:
                firstRow = False
                for lang in row[1:]:
                    languages.append(lang)
                TRANSLATIONS['languages'] = languages
                nbCols = len(languages)
            else:
                text = row[0]
                TRANSLATIONS[text] = {}
                for numCol in range(nbCols):
                    lang = languages[numCol]
                    translation = row[numCol + 1]
                    TRANSLATIONS[text][lang] = translation
        bge.logic.globalDict['languages'] = languages
    except:
        print('Failed to open {0}'.format(csvFileName))
    finally:
        theFile.close()


