#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Name:         server.py
# Author:       Pascal Peter
# Copyright:    (C) 2003-2012 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    programme serveur pour un jeu en réseau
    d'après Swinnen : http://inforef.be/swi/
"""

# importation des modules utiles :
from __future__ import print_function

import sys
import socket
import threading
import time



# **************************************************
#       POUR FONCTIONNER AVEC PYTHON 2.x OU 3.x
# **************************************************
PYTHONVERSION = sys.version_info[0] * 10 + sys.version_info[1]

def u(text):
    """
    retourne une version unicode de text
    """
    if PYTHONVERSION >= 30:
        try:
            if isinstance(text, str):
                return text
            else:
                return str(text)
        except:
            print('ERROR unicode | {0} | {1}'.format(type(text), text))
            return text
    else:
        try:
            return unicode(text)
        except:
            if isinstance(text, str):
                return text.decode('utf-8')
            else:
                print('ERROR unicode | {0} | {1}'.format(type(text), text))
                return text



# **************************************************
#       RÉCUPÉRATION DE L'IP DU POSTE
# **************************************************
def findMyIp():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('google.com', 80))
        myIp = s.getsockname()[0]
        s.close()
    except:
        myIp = 'localhost'
    print('myIp :', myIp)
    return myIp



# **************************************************
#       PARAMÈTRES DE CONNEXION
# **************************************************
if 'localhost' in sys.argv:
    HOST = 'localhost'
else:
    HOST = findMyIp()
PORT = 5005
BUFFER = 1024



# **************************************************
#       DICTIONNAIRES DES CLIENTS CONNECTÉS
#       ET ÉTAT DU SERVEUR
# **************************************************
CLIENTS = {}
RUNNING = False
TIME = 0


# **************************************************
#       THREAD DE GESTION DES CLIENTS
# **************************************************
class ThreadClient(threading.Thread):
    """
    dérivation d'un objet thread pour gérer la connexion avec un client
    """
    def __init__(self, connexion, parent=None):
        threading.Thread.__init__(self)
        self.connexion = connexion
        self.parent = parent

    def run(self):
        # Dialogue avec le client :
        global CLIENTS, TIME
        # Chaque thread possède un nom
        threadClientName = self.getName()
        clientName = ''
        while True:
            time.sleep(0.01)
            if clientName == '':
                if threadClientName in CLIENTS:
                    clientName = CLIENTS[threadClientName][1]
            try:
                if PYTHONVERSION >= 30:
                    msgClient = self.connexion.recv(BUFFER).decode('Utf8')
                else:
                    msgClient = u(self.connexion.recv(BUFFER))
            except:
                msgClient = 'FIN'
            if not(msgClient) or (msgClient.upper() == 'FIN'):
                break
            if PYTHONVERSION >= 30:
                message = '{0}|{1}|{2}'.format(threadClientName, clientName, msgClient)
            else:
                message = u('{0}|{1}|').format(threadClientName, clientName)
                message = message + msgClient
            actualTime = int(time.time() - TIME)
            if actualTime > 1:
                TIME = int(time.time())
                clientsList = []
                for client in CLIENTS:
                    clientsList.append(CLIENTS[client][1])
                print('clients :', clientsList)
            if msgClient.upper() == 'TEST':
                others = []
                for client in CLIENTS:
                    user = CLIENTS[client][1]
                    if (user != '') and not(user in others):
                        others.append(user)
                msgOthers = 'OTHERS|'
                for other in others:
                    msgOthers = '{0}{1}|'.format(msgOthers, other)
                self.sendMessage(threadClientName, msgOthers)
            else:
                if 'CONNECT' in msgClient:
                    CLIENTS[threadClientName][1] = msgClient.split('|')[1]
                elif (CLIENTS[threadClientName][1] == '') and ('LOOP' in msgClient):
                    CLIENTS[threadClientName][1] = msgClient.split('|')[1]
                # Faire suivre le message à tous les autres clients :
                for client in CLIENTS:
                    # ne pas le renvoyer à l'émetteur
                    if client != threadClientName:
                        self.sendMessage(client, message)
        # Fermeture de la connexion :
        # on avertit les autres :
        for client in CLIENTS:
            message = '{0}|{1}|DISCONNECT'.format(threadClientName, clientName)
            # ne pas le renvoyer à l'émetteur
            if client != threadClientName:
                self.sendMessage(client, message)
        # couper la connexion côté serveur
        self.connexion.close()
        # supprimer son entrée dans le dictionnaire
        del CLIENTS[threadClientName]
        print('{0}|{1}|DISCONNECT'.format(threadClientName, clientName))
        # Le thread se termine ici

    def sendMessage(self, connexionName, message):
        ok = False
        try:
            connexion = CLIENTS[connexionName][0]
            ok = True
            if PYTHONVERSION >= 30:
                connexion.send(message.encode('Utf8'))
            else:
                connexion.send(message.encode('Utf8'))
        except:
            if ok == False:
                print('PB sendMessage connexion')
            else:
                print('PB sendMessage send', connexionName, CLIENTS[connexionName][1])



# **************************************************
#       CLASSE SERVEUR
# **************************************************
class LanServer():
    def __init__(self, host=HOST):
        # Initialisation du serveur - Mise en place du socket :
        self.mySocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.mySocket.bind((host, PORT))
        except socket.error:
            print('SERVER SOCKET ERROR')
            return None
        print('SERVER READY ...')
        self.mySocket.listen(5)
        self.running = False

    def loop(self):
        global CLIENTS
        connexion, adresse = self.mySocket.accept()
        # Créer un nouvel objet thread pour gérer la connexion :
        threadClient = ThreadClient(connexion, self)
        threadClient.start()
        # Mémoriser la connexion dans le dictionnaire :
        # identifiant du thread
        threadClientName = threadClient.getName()
        CLIENTS[threadClientName] = [connexion, '']
        print('{0} : adresse IP {1}, port {2}.'.format(threadClientName, adresse[0], adresse[1]))



# **************************************************
#       PROGRAMME PRINCIPAL
# **************************************************
def main():
    global RUNNING
    lanServer = LanServer()
    RUNNING = True
    while RUNNING:
        lanServer.loop()


if __name__ == "__main__":
    main()
